//ppmc dac functions

#include "ppmc.h"


/*
VoltToCount(double volts)  converts a voltage to a
number of counts to output to the dac

  output to DAC word works like:

  0xFFFF -> +10 V
  0x8000 ->   0 V
  0x0000 -> -10 V
  */


__inline unsigned short VoltToCounts(double volts)
{
	long nCounts;
	nCounts = (long) (((volts / 10.0) * 0x7FFF)+0x8000);

	if (nCounts > 0xffff)
    {
      nCounts = 0xffff;
    }
	if (nCounts < 0 )
    {
      nCounts = 0;
    }
	return (unsigned short)nCounts;
}

int RawDAC(int nAxis, double volts)
{
  

  if ( (nAxis > PPMC_MAX_AXIS) || (nAxis < 0) )        /* is axis within range? */
    return -1;

  /* convert -10.0 -> 0,0 to 0x8000 10 -> 0xFFFF */
  
  if (nAxis <=3)
   {
    SelWrt16(DAC_0 + (nAxis << 1), VoltToCounts(volts));
	SelWrt(DAC_WRITE_0, 0);
   }
  else {
    SelWrt16(DAC_4 + ((nAxis-4) << 1), VoltToCounts(volts));
	SelWrt(DAC_WRITE_1, 0);
  }
	
	
  return 0;
}

short DACInit(void)
{
	int utl_count;
	double volts[PPMC_MAX_AXIS] ;
	for(utl_count=0;utl_count<PPMC_MAX_AXIS;utl_count++)
	{
		volts[utl_count]=0;
	}
	ppmcDacWriteAll(PPMC_MAX_AXIS,volts);
	SelWrt(DAC_MODE_0,1); // select strobe to 'manual'
	SelWrt(DAC_WRITE_0,1);    // strobe the DACs
       	SelWrt(DAC_WRITE_0,0);    // reset strobe
       	SelWrt(DAC_MODE_0,0);     // sel strobe to INTR/ line
	return 0;
}

short DACQuit(void)
{
	   return DACInit();  //init sets all outputs to zero
}
int ppmcDacNum(void)
{
	return PPMC_MAX_AXIS;
}

int ppmcDacWrite(int dac, double volts)
{
  return RawDAC(dac, volts);
}

int ppmcDacWriteAll(int max, double * volts)
{
	int t;
  int smax;
 
  /* clip smax to max supported-- if they want more, ignore */
  if (max > PPMC_MAX_AXIS) {
    smax = PPMC_MAX_AXIS;
  }
  else {
    smax = max;
  }
#ifndef PPMC_8_AXIS
  //take advantage of auto increment of address pointer in ppmc
	SelAddr(DAC_0);
	for (t = 0; t < smax; t++)
	{
  /*	  printf(" dac channel %d volts %12.4f   digital # %d\n",
	  t,volts[t],VoltToCounts(volts[t]));  */
		WriteData16(VoltToCounts(volts[t]));
    }
	/*	printf("\n");  */
	WriteData( 0);//output data to analog
#else
	SelAddr(DAC_0);
	if(smax<=4)
	{
		for (t = 0; t < smax; t++)
		{
			 WriteData16(VoltToCounts(volts[t]));
		}
		WriteData( 0);
		return 0;
	}
	else
	{
		for (t = 0; t < 4; t++)
		{
			WriteData16(VoltToCounts(volts[t]));
 
		}
	}

	WriteData( 0);//output data to analog
	setaddr(DAC_1);
	for (t = 4; t < smax; t++)
	{
		WriteData16(VoltToCounts(volts[t]));
    }

	WriteData( 0);//output data to analog

#endif
  

  return 0;
  
}
